package com.jzdsh.common.database;

import com.jfinal.kit.PathKit;
import com.jfinal.kit.Prop;
import com.jfinal.kit.PropKit;
import com.jfinal.plugin.activerecord.dialect.MysqlDialect;
import com.jfinal.plugin.activerecord.generator.Generator;
import com.jfinal.plugin.activerecord.generator.ModelGenerator;
import com.jfinal.plugin.druid.DruidPlugin;
import org.apache.commons.lang3.StringUtils;

import javax.sql.DataSource;

public class _Generator {
// region 生成MODEL

    /**
     * 生成MODEL 这里会根据不同的数据库生成MODEL类文件至不同的文件夹中
     *
     * @param args
     */
    public static void main(String[] args) {
        PropKit.clear();
        //PropKit.use("local_dev.properties");
        //generate(DbConfig.DbEnum.Spider);
        //generate(DatabaseEnum.SpiderNew);
    }

    /**
     * 生成实体类
     * @param dbModelPackageName db model的包名 for example ：com.chinaventure.spider.model
     */
    public static void generate(String dbModelPackageName) {
        Prop p = PropKit.use("config-dev.txt");
        generate(new DbConfig(p),dbModelPackageName);
    }

    /**
     * 生成数据库实体类
     * @param config 那个数据库
     * @param dbModelPackageName db model的包名 for example ：com.chinaventure.spider.model
     */
    public static void generate(DbConfig config,String dbModelPackageName) {
        //主要数据库包名
        String mainPackageName = dbModelPackageName + ".base";
        //次要要数据库包名
        String subPackageName = dbModelPackageName+"." + config.getConfigName() + ".base";

        String baseModelPackageName = StringUtils.equalsIgnoreCase(config.getConfigName(), "normal") ? mainPackageName : subPackageName;
        // base model 文件保存路径
        String baseModelOutputDir = PathKit.getWebRootPath() + "/src/main/java/" + baseModelPackageName.replace('.', '/');

        // model 所使用的包名 (MappingKit 默认使用的包名)
        String modelPackageName = baseModelPackageName.substring(0, baseModelPackageName.lastIndexOf('.'));
        // model 文件保存路径 (MappingKit 与 DataDictionary 文件默认保存路径)
        String modelOutputDir = baseModelOutputDir + "/..";

        // 数据源
        DruidPlugin dsMysql = DbConfig.getDruidPlugin(config);
        dsMysql.start();
        DataSource dataSource = dsMysql.getDataSource();

        // 创建生成器
        Generator generator = new Generator(dataSource,new _BaseModelGenerator(baseModelPackageName, baseModelOutputDir),new ModelGenerator(modelPackageName, baseModelPackageName, modelOutputDir));

        // 配置是否生成备注
        generator.setGenerateRemarks(true);

        // 设置数据库方言
        generator.setDialect(new MysqlDialect());

        // 设置是否生成链式 setter 方法
        generator.setGenerateChainSetter(true);

        // 添加不需要生成的表名
        generator.addExcludedTable("");

        // 设置是否在 Model 中生成 dao 对象
        generator.setGenerateDaoInModel(true);

        // 设置是否生成字典文件
        generator.setGenerateDataDictionary(true);

        // 设置需要被移除的表名前缀用于生成modelName。例如表名 "osc_user"，移除前缀 "osc_"后生成的model名为 "User"而非 OscUser
        generator.setRemovedTableNamePrefixes("t_");

        // 生成
        generator.generate();
    }

    // endregion
}
